@@ -53,7 +53,7 @@ SLUGIFY_HYPHENATE_RE = re.compile(r"[-\s]+")
 # NOTE(flaper87): The following globals are used by `mask_password`
 _SANITIZE_KEYS = ['adminPass', 'admin_pass', 'password', 'admin_password',
                   'auth_token', 'new_pass', 'auth_password', 'secret_uuid',
-                  'sys_pswd']
+                  'secret', 'sys_pswd', 'token', 'configdrive']
 
 # NOTE(ldbragst): Let's build a list of regex objects using the list of
 # _SANITIZE_KEYS we already have. This way, we only have to add the new key
@@ -119,6 +119,8 @@ def bool_from_string(subject, strict=False, default=False):
     ValueError which is useful when parsing values passed in from an API call.
     Strings yielding False are 'f', 'false', 'off', 'n', 'no', or '0'.
     """
+    if isinstance(subject, bool):
+        return subject
     if not isinstance(subject, six.string_types):
         subject = six.text_type(subject)
 
@@ -210,8 +212,21 @@ def to_slug(value, incoming=None, errors="strict"):
     return SLUGIFY_HYPHENATE_RE.sub("-", value)
 
 
-def mask_password(message, secret="***"):
-    """Replace password with 'secret' in message.
+# NOTE(dhellmann): Before submitting a patch to add a new argument to
+# this function to allow the caller to pass in "extra" or "additional"
+# or "replacement" patterns to be masked out, please note that we have
+# discussed that feature many times and always rejected it based on
+# the desire to have Oslo functions behave consistently across all
+# projects and *especially* to have security features work the same
+# way no matter where they are used. If every project adopted its own
+# set patterns for secret values, it would be very difficult to audit
+# the logging to ensure that everything is properly masked. So, please
+# either add your pattern to the module-level variables at the top of
+# this file or, even better, pick an existing pattern or key to use in
+# your application to ensure that the value is masked by this
+# function.
+def mask_password(message, secret="***"):  # nosec
+    """Replace password with *secret* in message.
 
     :param message: The string which includes security information.
     :param secret: value with which to replace passwords.
@@ -229,11 +244,32 @@ def mask_password(message, secret="***"):
     "'original_password' : '***'"
     >>> mask_password("u'original_password' :   u'aaaaa'")
     "u'original_password' :   u'***'"
+
+    .. versionadded:: 0.2
+
+    .. versionchanged:: 1.1
+       Replace also ``'auth_token'``, ``'new_pass'`` and ``'auth_password'``
+       keys.
+
+    .. versionchanged:: 1.1.1
+       Replace also ``'secret_uuid'`` key.
+
+    .. versionchanged:: 1.5
+       Replace also ``'sys_pswd'`` key.
+
+    .. versionchanged:: 2.6
+       Replace also ``'token'`` key.
+
+    .. versionchanged:: 2.7
+       Replace also ``'secret'`` key.
+
+    .. versionchanged:: 3.4
+       Replace also ``'configdrive'`` key.
     """
 
     try:
         message = six.text_type(message)
-    except UnicodeDecodeError:
+    except UnicodeDecodeError:  # nosec
         # NOTE(jecarey): Temporary fix to handle cases where message is a
         # byte string. A better solution will be provided in Kilo.
         pass
@@ -254,14 +290,121 @@ def mask_password(message, secret="***"):
     return message
 
 
+def mask_dict_password(dictionary, secret="***"):  # nosec
+    """Replace password with *secret* in a dictionary recursively.
+
+    :param dictionary: The dictionary which includes secret information.
+    :param secret: value with which to replace secret information.
+    :returns: The dictionary with string substitutions.
+
+    A dictionary (which may contain nested dictionaries) contains
+    information (such as passwords) which should not be revealed, and
+    this function helps detect and replace those with the 'secret'
+    provided (or '***' if none is provided).
+
+    Substitution is performed in one of three situations:
+
+    If the key is something that is considered to be indicative of a
+    secret, then the corresponding value is replaced with the secret
+    provided (or '***' if none is provided).
+
+    If a value in the dictionary is a string, then it is masked
+    using the mask_password() function.
+
+    Finally, if a value is a dictionary, this function will
+    recursively mask that dictionary as well.
+
+    For example:
+
+    >>> mask_dict_password({'password': 'd81juxmEW_',
+    >>>                     'user': 'admin',
+    >>>                     'home-dir': '/home/admin'},
+    >>>                     '???')
+    {'password': '???', 'user': 'admin', 'home-dir': '/home/admin'}
+
+    For example (the value is masked using mask_password())
+
+    >>> mask_dict_password({'password': '--password d81juxmEW_',
+    >>>                     'user': 'admin',
+    >>>                     'home-dir': '/home/admin'},
+    >>>                     '???')
+    {'password': '--password ???', 'user': 'admin',
+     'home-dir': '/home/admin'}
+
+
+    For example (a nested dictionary is masked):
+
+    >>> mask_dict_password({"nested": {'password': 'd81juxmEW_',
+    >>>                     'user': 'admin',
+    >>>                     'home': '/home/admin'}},
+    >>>                     '???')
+    {"nested": {'password': '???', 'user': 'admin', 'home': '/home/admin'}}
+
+    .. versionadded:: 3.4
+
+    """
+
+    if not isinstance(dictionary, dict):
+        raise TypeError("Expected a dictionary, got %s instead."
+                        % type(dictionary))
+
+    out = {}
+
+    for k, v in dictionary.items():
+        if isinstance(v, dict):
+            v = mask_dict_password(v, secret=secret)
+        elif k in _SANITIZE_KEYS:
+            v = secret
+        elif isinstance(v, six.string_types):
+            v = mask_password(v, secret=secret)
+
+        out[k] = v
+
+    return out
+
+
 def is_int_like(val):
     """Check if a value looks like an integer with base 10.
 
     :param val: Value to verify
     :type val: string
     :returns: bool
+
+    .. versionadded:: 1.1
     """
     try:
         return six.text_type(int(val)) == six.text_type(val)
     except (TypeError, ValueError):
         return False
+
+
+def check_string_length(value, name=None, min_length=0, max_length=None):
+    """Check the length of specified string.
+
+    :param value: the value of the string
+    :param name: the name of the string
+    :param min_length: the min_length of the string
+    :param max_length: the max_length of the string
+    :raises TypeError, ValueError: For any invalid input.
+
+    .. versionadded:: 3.7
+    """
+    if name is None:
+        name = value
+
+    if not isinstance(value, six.string_types):
+        msg = _("%s is not a string or unicode") % name
+        raise TypeError(msg)
+
+    length = len(value)
+    if length < min_length:
+        msg = _("%(name)s has %(length)s characters, less than "
+                "%(min_length)s.") % {'name': name, 'length': length,
+                                      'min_length': min_length}
+        raise ValueError(msg)
+
+    if max_length and length > max_length:
+        msg = _("%(name)s has %(length)s characters, more than "
+                "%(max_length)s.") % {'name': name, 'length': length,
+                                      'max_length': max_length}
+        raise ValueError(msg)
